@@ -85,6 +85,7 @@ typedef struct {
 	GError *reload_error;
 
 	GSList *pending;        /* ordered list of pending property updates. */
+	GPtrArray *proxies;
 } NMObjectPrivate;
 
 enum {
@@ -596,7 +597,18 @@ handle_object_property (NMObject *self, const char *property_name, GVariant *val
 	if (!object) {
 		/* This is a server bug -- a dangling object path for an object
 		 * that does not exist. */
-		g_warning ("No object known for %s", path);
+		/* XXX: We've ignored this before and the server hits the condition
+		 * more often that it should. Given we're able to recover from
+		 * ther error, let's lower the severity of the log message to
+		 * avoid unnecessarily bothering the user. This can be removed
+		 * once the issue is fixed on the server. */
+#if NM_MORE_ASSERTS
+#define __nm_log_debug g_warning
+#else
+#define __nm_log_debug g_debug
+#endif
+		__nm_log_debug ("No object known for %s", path);
+#undef __nm_log_debug
 		return FALSE;
 	}
 
@@ -928,7 +940,7 @@ _nm_object_register_properties (NMObject *object,
 	proxy = _nm_object_get_proxy (object, interface);
 	g_signal_connect (proxy, "g-properties-changed",
 		          G_CALLBACK (properties_changed), object);
-	g_object_unref (proxy);
+	g_ptr_array_add (priv->proxies, proxy);
 
 	instance = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->property_tables = g_slist_prepend (priv->property_tables, instance);
@@ -1177,6 +1189,7 @@ nm_object_async_initable_iface_init (GAsyncInitableIface *iface)
 static void
 nm_object_init (NMObject *object)
 {
+	NM_OBJECT_GET_PRIVATE (object)->proxies = g_ptr_array_new ();
 }
 
 static void
@@ -1189,6 +1202,8 @@ set_property (GObject *object, guint prop_id,
 	case PROP_DBUS_OBJECT:
 		/* Construct only */
 		priv->object = g_value_dup_object (value);
+		if (!priv->object)
+			g_return_if_reached ();
 		break;
 	case PROP_DBUS_OBJECT_MANAGER:
 		/* Construct only */
@@ -1229,6 +1244,7 @@ static void
 dispose (GObject *object)
 {
 	NMObjectPrivate *priv = NM_OBJECT_GET_PRIVATE (object);
+	guint i;
 
 	nm_clear_g_source (&priv->notify_id);
 
@@ -1240,6 +1256,17 @@ dispose (GObject *object)
 	g_clear_object (&priv->object);
 	g_clear_object (&priv->object_manager);
 
+	if (priv->proxies) {
+		for (i = 0; i < priv->proxies->len; i++) {
+			g_signal_handlers_disconnect_by_func (priv->proxies->pdata[i],
+			                                      properties_changed,
+			                                      object);
+			g_object_unref (priv->proxies->pdata[i]);
+		}
+		g_ptr_array_free (priv->proxies, TRUE);
+		priv->proxies = NULL;
+	}
+
 	G_OBJECT_CLASS (nm_object_parent_class)->dispose (object);
 }
 
