@@ -27,8 +27,8 @@
 
 #include "nm-bluez-common.h"
 #include "nm-bluez-device.h"
-#include "nm-device-private.h"
-#include "ppp-manager/nm-ppp-manager.h"
+#include "devices/nm-device-private.h"
+#include "ppp/nm-ppp-manager.h"
 #include "nm-setting-connection.h"
 #include "nm-setting-bluetooth.h"
 #include "nm-setting-cdma.h"
@@ -36,15 +36,13 @@
 #include "nm-setting-serial.h"
 #include "nm-setting-ppp.h"
 #include "NetworkManagerUtils.h"
-#include "nm-bt-enum-types.h"
 #include "nm-utils.h"
 #include "nm-bt-error.h"
-#include "nm-bt-enum-types.h"
-#include "nm-platform.h"
+#include "platform/nm-platform.h"
 
-#include "nmdbus-device-bt.h"
+#include "introspection/org.freedesktop.NetworkManager.Device.Bluetooth.h"
 
-#include "nm-device-logging.h"
+#include "devices/nm-device-logging.h"
 _LOG_DECLARE_SELF(NMDeviceBt);
 
 #define MM_DBUS_SERVICE   "org.freedesktop.ModemManager1"
@@ -485,8 +483,8 @@ modem_prepare_result (NMModem *modem,
 			 * the device to be auto-activated anymore, which would risk locking
 			 * the SIM if the incorrect PIN continues to be used.
 			 */
-			nm_device_set_autoconnect (device, FALSE);
 			_LOGI (LOGD_MB, "disabling autoconnect due to failed SIM PIN");
+			nm_device_set_autoconnect_intern (device, FALSE);
 		}
 
 		nm_device_state_changed (device, NM_DEVICE_STATE_FAILED, reason);
@@ -582,10 +580,12 @@ modem_cleanup (NMDeviceBt *self)
 
 static void
 modem_state_cb (NMModem *modem,
-                NMModemState new_state,
-                NMModemState old_state,
+                int new_state_i,
+                int old_state_i,
                 gpointer user_data)
 {
+	NMModemState new_state = new_state_i;
+	NMModemState old_state = old_state_i;
 	NMDevice *device = NM_DEVICE (user_data);
 	NMDeviceState dev_state = nm_device_get_state (device);
 
