@@ -319,7 +319,8 @@ pk_call_cb (GObject *object, GAsyncResult *result, gpointer user_data)
 {
 	AuthCall *call = user_data;
 	GError *error = NULL;
-	gboolean is_authorized, is_challenge;
+	gboolean is_authorized = FALSE, is_challenge = FALSE;
+	guint call_result = NM_AUTH_CALL_RESULT_UNKNOWN;
 
 	nm_auth_manager_polkit_authority_check_authorization_finish (NM_AUTH_MANAGER (object),
 	                                                             result,
@@ -336,17 +337,11 @@ pk_call_cb (GObject *object, GAsyncResult *result, gpointer user_data)
 	}
 
 	if (error) {
+		/* Don't ruin the chain. Just leave the result unknown. */
 		nm_log_warn (LOGD_CORE, "error requesting auth for %s: %s",
 		             call->permission, error->message);
-
-		if (!call->chain->error) {
-			call->chain->error = error;
-			error = NULL;
-		} else
-			g_clear_error (&error);
+		g_clear_error (&error);
 	} else {
-		guint call_result = NM_AUTH_CALL_RESULT_UNKNOWN;
-
 		if (is_authorized) {
 			/* Caller has the permission */
 			call_result = NM_AUTH_CALL_RESULT_YES;
@@ -355,10 +350,10 @@ pk_call_cb (GObject *object, GAsyncResult *result, gpointer user_data)
 			call_result = NM_AUTH_CALL_RESULT_AUTH;
 		} else
 			call_result = NM_AUTH_CALL_RESULT_NO;
-
-		nm_auth_chain_set_data (call->chain, call->permission, GUINT_TO_POINTER (call_result), NULL);
 	}
 
+	nm_auth_chain_set_data (call->chain, call->permission, GUINT_TO_POINTER (call_result), NULL);
+
 	auth_call_complete (call);
 }
 #endif
@@ -399,8 +394,8 @@ nm_auth_chain_add_call (NMAuthChain *self,
 		                                                      call);
 #else
 		if (!call->chain->error) {
-			call->chain->error = g_error_new_literal (DBUS_GERROR,
-			                                          DBUS_GERROR_FAILED,
+			call->chain->error = g_error_new_literal (NM_MANAGER_ERROR,
+			                                          NM_MANAGER_ERROR_FAILED,
 			                                          "Polkit support is disabled at compile time");
 		}
 		call->call_idle_id = g_idle_add ((GSourceFunc) auth_call_complete, call);
